# coding: utf-8
#
# Copyright (c) 2015 Cossack Labs Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import warnings
from ctypes import cdll, c_int, byref, create_string_buffer, string_at
from ctypes.util import find_library

from .exception import ThemisError
from .exception import THEMIS_CODES

themis = cdll.LoadLibrary(find_library('themis'))


class SCellSeal(object):
    def __init__(self, key):
        if not key:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Seal) failed creating")
        self.key = key

    def encrypt(self, message, context=None):
        context_length = len(context) if context else 0
        encrypted_message_length = c_int(0)
        if themis.themis_secure_cell_encrypt_seal(
                self.key, len(self.key), context, context_length,
                message, len(message), None,
                byref(encrypted_message_length)) != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Seal) failed encrypting")
        encrypted_message = create_string_buffer(encrypted_message_length.value)
        if themis.themis_secure_cell_encrypt_seal(
                self.key, len(self.key), context, context_length,
                message, len(message), encrypted_message,
                byref(encrypted_message_length)) != THEMIS_CODES.SUCCESS:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Seal) failed encrypting")

        return string_at(encrypted_message, encrypted_message_length.value)

    def decrypt(self, message, context=None):
        context_length = len(context) if context else 0
        decrypted_message_length = c_int(0)
        if themis.themis_secure_cell_decrypt_seal(
                self.key, len(self.key), context, context_length,
                message, len(message), None,
                byref(decrypted_message_length)) != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Seal) failed decrypting")
        decrypted_message = create_string_buffer(decrypted_message_length.value)
        if themis.themis_secure_cell_decrypt_seal(
                self.key, len(self.key), context, context_length,
                message, len(message), decrypted_message,
                byref(decrypted_message_length)) != THEMIS_CODES.SUCCESS:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Seal) failed decrypting")

        return string_at(decrypted_message, decrypted_message_length.value)


class SCellTokenProtect(object):
    def __init__(self, key):
        if not key:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Token Protect) failed creating")
        self.key = key

    def encrypt(self, message, context=None):
        context_length_ = len(context) if context else 0
        encrypted_message_length = c_int(0)
        context_length = c_int(0)
        if themis.themis_secure_cell_encrypt_token_protect(
                self.key, len(self.key), context, context_length_,
                message, len(message), None, byref(context_length), None,
                byref(encrypted_message_length)) != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Token Protect) failed encrypting")
        encrypted_message = create_string_buffer(encrypted_message_length.value)
        token = create_string_buffer(context_length.value)
        if themis.themis_secure_cell_encrypt_token_protect(
                self.key, len(self.key), context, context_length_,
                message, len(message), token, byref(context_length),
                encrypted_message, byref(encrypted_message_length)) != THEMIS_CODES.SUCCESS:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Token Protect) failed encrypting")
        return (string_at(encrypted_message, encrypted_message_length.value),
                string_at(token, context_length))

    def decrypt(self, message, token, context=None):
        context_length = len(context) if context else 0
        decrypted_message_length = c_int(0)
        res = themis.themis_secure_cell_decrypt_token_protect(
            self.key, len(self.key), context, context_length,
            message, len(message), token, len(token), None,
            byref(decrypted_message_length))
        if res != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Token Protect) failed decrypting")
        decrypted_message = create_string_buffer(decrypted_message_length.value)
        if themis.themis_secure_cell_decrypt_token_protect(
                self.key, len(self.key), context, context_length,
                message, len(message), token, len(token), decrypted_message,
                byref(decrypted_message_length)) != THEMIS_CODES.SUCCESS:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Token Protect) failed decrypting")

        return string_at(decrypted_message, decrypted_message_length.value)


class SCellContextImprint(object):
    def __init__(self, key):
        if not key:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Context Imprint) failed creating")
        self.key = key

    def encrypt(self, message, context):
        encrypted_message_length = c_int(0)
        if themis.themis_secure_cell_encrypt_context_imprint(
                self.key, len(self.key), message, len(message),
                context, len(context), None,
                byref(encrypted_message_length)) != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Context Imprint) failed encrypting")
        encrypted_message = create_string_buffer(encrypted_message_length.value)
        if themis.themis_secure_cell_encrypt_context_imprint(
                self.key, len(self.key), message, len(message),
                context, len(context), encrypted_message,
                byref(encrypted_message_length)) != THEMIS_CODES.SUCCESS:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Context Imprint) failed encrypting")

        return string_at(encrypted_message, encrypted_message_length.value)

    def decrypt(self, message, context):
        decrypted_message_length = c_int(0)
        if themis.themis_secure_cell_decrypt_context_imprint(
                self.key, len(self.key), message, len(message),
                context, len(context), None,
                byref(decrypted_message_length)) != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Context Imprint) failed decrypting")
        decrypted_message = create_string_buffer(decrypted_message_length.value)
        if themis.themis_secure_cell_decrypt_context_imprint(
                self.key, len(self.key), message, len(message),
                context, len(context), decrypted_message,
                byref(decrypted_message_length)) != THEMIS_CODES.SUCCESS:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Cell (Context Imprint) failed decrypting")

        return string_at(decrypted_message, decrypted_message_length.value)


class scell_seal(SCellSeal):
    def __init__(self, *args, **kwargs):
        warnings.warn("scell_seal is deprecated in favor of SCellSeal.")
        super(scell_seal, self).__init__(*args, **kwargs)


class scell_token_protect(SCellTokenProtect):
    def __init__(self, *args, **kwargs):
        warnings.warn("scell_token_protect is deprecated in favor of "
                      "SCellTokenProtect.")
        super(scell_token_protect, self).__init__(*args, **kwargs)


class scell_context_imprint(SCellContextImprint):
    def __init__(self, *args, **kwargs):
        warnings.warn("scell_context_imprint is deprecated in favor of "
                      "SCellContextImprint.")
        super(scell_context_imprint, self).__init__(*args, **kwargs)



#
# Copyright (c) 2015 Cossack Labs Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import warnings
from ctypes import cdll, create_string_buffer, c_int, string_at, byref
from ctypes.util import find_library
from enum import IntEnum

from .exception import ThemisError, THEMIS_CODES

themis = cdll.LoadLibrary(find_library('themis'))


class SMessage(object):
    def __init__(self, private_key, peer_public_key):
        if not private_key:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Message: missing private key")
        if not peer_public_key:
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Message: missing public key")
        if not _private_key(private_key):
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Message: invalid private key")
        if not _public_key(peer_public_key):
            raise ThemisError(THEMIS_CODES.FAIL,
                              "Secure Message: invalid public key")
        self.private_key = private_key
        self.peer_public_key = peer_public_key

    def wrap(self, message):
        encrypted_message_length = c_int(0)
        res = themis.themis_secure_message_encrypt(
            self.private_key, len(self.private_key),
            self.peer_public_key, len(self.peer_public_key),
            message, len(message), None, byref(encrypted_message_length))
        if res != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise ThemisError(res, "Secure Message failed to encrypt")
        encrypted_message = create_string_buffer(encrypted_message_length.value)
        res = themis.themis_secure_message_encrypt(
            self.private_key, len(self.private_key),
            self.peer_public_key, len(self.peer_public_key),
            message, len(message), encrypted_message,
            byref(encrypted_message_length))
        if res != THEMIS_CODES.SUCCESS:
            raise ThemisError(res, "Secure Message failed to encrypt")

        return string_at(encrypted_message, encrypted_message_length.value)

    def unwrap(self, message):
        plain_message_length = c_int(0)
        res = themis.themis_secure_message_decrypt(
            self.private_key, len(self.private_key),
            self.peer_public_key, len(self.peer_public_key),
            message, len(message), None, byref(plain_message_length))
        if res != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise ThemisError(res, "Secure Message failed to decrypt")
        plain_message = create_string_buffer(plain_message_length.value)
        res = themis.themis_secure_message_decrypt(
            self.private_key, len(self.private_key),
            self.peer_public_key, len(self.peer_public_key),
            message, len(message), plain_message, byref(plain_message_length))
        if res != THEMIS_CODES.SUCCESS:
            raise ThemisError(res, "Secure Message failed to decrypt")
        return string_at(plain_message, plain_message_length.value)


def ssign(private_key, message):
    if not private_key:
        raise ThemisError(THEMIS_CODES.FAIL,
                          "Secure Message: missing private key")
    if not _private_key(private_key):
        raise ThemisError(THEMIS_CODES.FAIL,
                          "Secure Message: invalid private key")
    encrypted_message_length = c_int(0)
    res = themis.themis_secure_message_sign(
        private_key, len(private_key), message, len(message),
        None, byref(encrypted_message_length))
    if res != THEMIS_CODES.BUFFER_TOO_SMALL:
        raise ThemisError(res, "Secure Message failed to sign")
    encrypted_message = create_string_buffer(encrypted_message_length.value)
    res = themis.themis_secure_message_sign(
        private_key, len(private_key), message, len(message),
        encrypted_message, byref(encrypted_message_length))
    if res != THEMIS_CODES.SUCCESS:
        raise ThemisError(res, "Secure Message failed to sign")
    return string_at(encrypted_message, encrypted_message_length.value)


def sverify(public_key, message):
    if not public_key:
        raise ThemisError(THEMIS_CODES.FAIL,
                          "Secure Message: missing public key")
    if not _public_key(public_key):
        raise ThemisError(THEMIS_CODES.FAIL,
                          "Secure Message: invalid public key")
    plain_message_length = c_int(0)
    res = themis.themis_secure_message_verify(
        public_key, len(public_key), message, len(message),
        None, byref(plain_message_length))
    if res != THEMIS_CODES.BUFFER_TOO_SMALL:
        raise ThemisError(res, "Secure Message failed to verify")
    plain_message = create_string_buffer(plain_message_length.value)
    res = themis.themis_secure_message_verify(
        public_key, len(public_key), message, len(message),
        plain_message, byref(plain_message_length))
    if res != THEMIS_CODES.SUCCESS:
        raise ThemisError(res, "Secure Message failed to verify")

    return string_at(plain_message, plain_message_length.value)


class THEMIS_KEY(IntEnum):
    INVALID = 0
    RSA_PRIVATE = 1
    RSA_PUBLIC = 2
    EC_PRIVATE = 3
    EC_PUBLIC = 4


def _public_key(key):
    res = themis.themis_is_valid_asym_key(key, len(key))
    if res != THEMIS_CODES.SUCCESS:
        return False
    kind = themis.themis_get_asym_key_kind(key, len(key))
    return kind in [THEMIS_KEY.RSA_PUBLIC, THEMIS_KEY.EC_PUBLIC]


def _private_key(key):
    res = themis.themis_is_valid_asym_key(key, len(key))
    if res != THEMIS_CODES.SUCCESS:
        return False
    kind = themis.themis_get_asym_key_kind(key, len(key))
    return kind in [THEMIS_KEY.RSA_PRIVATE, THEMIS_KEY.EC_PRIVATE]


class smessage(SMessage):
    def __init__(self, *args, **kwargs):
        warnings.warn("smessage is deprecated in favor of SMessage.")
        super(smessage, self).__init__(*args, **kwargs)



#
# Copyright (c) 2015 Cossack Labs Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import warnings
from ctypes import cdll, c_int, create_string_buffer, byref, string_at
from ctypes.util import find_library

from .exception import ThemisError, THEMIS_CODES

themis = cdll.LoadLibrary(find_library('themis'))


class KEY_PAIR_TYPE(object):
    EC = 'EC'
    RSA = 'RSA'
    CHOICES = (EC, RSA)


class GenerateKeyPair(object):
    def __init__(self, alg):
        self.private_key_length = c_int(4096)
        self.public_key_length = c_int(4096)
        self.private_key = create_string_buffer(self.private_key_length.value)
        self.public_key = create_string_buffer(self.public_key_length.value)

        if alg not in KEY_PAIR_TYPE.CHOICES:
            raise ThemisError(THEMIS_CODES.FAIL, "Incorrect KeyPair type")

        if alg == KEY_PAIR_TYPE.EC:
            if themis.themis_gen_ec_key_pair(
                    self.private_key, byref(self.private_key_length),
                    self.public_key, byref(self.public_key_length)) != 0:
                raise ThemisError(THEMIS_CODES.FAIL,
                                  "Themis failed generating EC KeyPair")
        elif alg == KEY_PAIR_TYPE.RSA:
            if themis.themis_gen_rsa_key_pair(
                    self.private_key, byref(self.private_key_length),
                    self.public_key, byref(self.public_key_length)) != 0:
                raise ThemisError(THEMIS_CODES.FAIL,
                                  "Themis failed generating RSA KeyPair")

    def export_private_key(self):
        return string_at(self.private_key, self.private_key_length.value)

    def export_public_key(self):
        return string_at(self.public_key, self.public_key_length.value)


class themis_gen_key_pair(GenerateKeyPair):
    def __init__(self, *args, **kwargs):
        warnings.warn("themis_gen_key_pair is deprecated in favor of "
                      "GenerateKeyPair.")
        super(themis_gen_key_pair, self).__init__(*args, **kwargs)



#!/usr/bin/env python
# coding: utf-8
#
# Copyright (c) 2015 Cossack Labs Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import warnings
import ctypes
from ctypes.util import find_library
from collections import deque
from abc import abstractmethod

from . import exception as exception
from .exception import THEMIS_CODES, ThemisError

themis = ctypes.cdll.LoadLibrary(find_library('themis'))

ON_GET_PUBLIC_KEY = ctypes.CFUNCTYPE(
    ctypes.c_int, ctypes.POINTER(ctypes.c_byte),
    ctypes.c_size_t, ctypes.POINTER(ctypes.c_byte),
    ctypes.c_size_t, ctypes.POINTER(ctypes.py_object))
ON_SEND_DATA = ctypes.CFUNCTYPE(
    ctypes.c_ssize_t, ctypes.POINTER(ctypes.c_byte),
    ctypes.c_size_t, ctypes.POINTER(ctypes.py_object))
ON_RECEIVE_DATA = ctypes.CFUNCTYPE(
    ctypes.c_ssize_t, ctypes.POINTER(ctypes.c_byte),
    ctypes.c_size_t, ctypes.POINTER(ctypes.py_object))
ON_STATE_CHANGE = ctypes.CFUNCTYPE(ctypes.c_int, ctypes.c_int, ctypes.c_void_p)


class TransportStruct(ctypes.Structure):
    # set of callbacks
    _fields_ = [
        # use for "sending" data.
        # if set to None - secure session methods send/receive not usable
        ("send_data", ON_SEND_DATA),
        # use for "receiving" data.
        # if set to None - secure session methods send/receive not usable
        ("receive_data", ON_RECEIVE_DATA),
        # not used for in current version
        ("state_changed", ON_STATE_CHANGE),
        # [necessary] use for getting peer public key by it ID
        # (see ssession.__init__ method).
        ("get_public_key_for_id", ON_GET_PUBLIC_KEY),
        # some user_data, that will be passed to any of callbacks
        ("user_data", ctypes.POINTER(ctypes.py_object))]


ssession_create = themis.secure_session_create
ssession_create.restype = ctypes.POINTER(ctypes.c_int)
ssession_create.argtypes = [ctypes.c_void_p, ctypes.c_size_t,
                            ctypes.c_void_p, ctypes.c_size_t,
                            ctypes.POINTER(TransportStruct)]


def on_send(data, data_length, user_data):
    try:
        user_data[0].send(ctypes.string_at(data, data_length))
    except Exception:
        return THEMIS_CODE.NETWORK_ERROR
    return data_length


def on_receive(data, data_length, user_data):
    try:
        received_data = user_data[0].receive(data_length)
    except Exception as e:
        return THEMIS_CODE.NETWORK_ERROR
    ctypes.memmove(data, received_data, len(received_data))
    return len(received_data)


def on_get_pub_key(user_id, id_length, key_buffer, key_buffer_length,
                   user_data):
    real_user_id = ctypes.string_at(user_id, id_length)
    pub_key = user_data[0].get_pub_key_by_id(real_user_id)
    ctypes.memmove(key_buffer, pub_key, len(pub_key))
    return 0


def on_change_status(buffer_length, user_data):
    return 0


on_send_ = ON_SEND_DATA(on_send)
on_receive_ = ON_RECEIVE_DATA(on_receive)
on_change_status_ = ON_STATE_CHANGE(on_change_status)
on_get_pub_key_ = ON_GET_PUBLIC_KEY(on_get_pub_key)
lp_conn_type = ctypes.POINTER(ctypes.py_object)


class SString(bytes):
    def __new__(cls, value):
        obj = bytes.__new__(cls, value)
        obj.is_control = False
        return obj

    def set_control(self):
        self.is_control = True

    def unset_control(self):
        self.is_control = False

    def is_control(self):
        return self.is_control

    def __str__(self):
        return bytes.__str__(self)


class SSession(object):
    def __init__(self, user_id, sign_key, transport):
        # user_id - user identification ("server" for example)
        # sign_key - private key of session owner
        # transport - refference for transport_t object.
        self.session_ctx = ctypes.POINTER(ctypes.c_int)
        if transport is None:
            self.session_ctx = ssession_create(
                ctypes.byref(ctypes.create_string_buffer(user_id)),
                len(user_id),
                ctypes.byref(ctypes.create_string_buffer(sign_key)),
                len(sign_key),
                0)
        else:
            self.lp_conn = lp_conn_type(ctypes.py_object(transport))
            self.transport_ = TransportStruct(
                on_send_, on_receive_, on_change_status_, on_get_pub_key_,
                self.lp_conn)
            self.session_ctx = ssession_create(
                ctypes.byref(ctypes.create_string_buffer(user_id)),
                len(user_id),
                ctypes.byref(ctypes.create_string_buffer(sign_key)),
                len(sign_key),
                ctypes.byref(self.transport_))
        if self.session_ctx is None:
            raise exception.ThemisError(THEMIS_CODES.FAIL,
                                        "Secure Session failed creating")

    def __del__(self):
        themis.secure_session_destroy(self.session_ctx)

    def connect(self):
        res = themis.secure_session_connect(self.session_ctx)
        if res != THEMIS_CODES.SUCCESS:
            raise exception.ThemisError(res, "Secure Session failed connecting")

    def send(self, message):
        send_message = ctypes.create_string_buffer(message)
        res = themis.secure_session_send(
            self.session_ctx, ctypes.byref(send_message), len(message))
        if res == THEMIS_CODES.NETWORK_ERROR:
            raise exception.ThemisError(res, "Secure Session failed sending")

        return res

    def receive(self):
        message = ctypes.create_string_buffer(1024)
        message_length = ctypes.c_size_t(1024)
        res = themis.secure_session_receive(self.session_ctx, message,
                                            message_length)
        if res == THEMIS_CODES.NETWORK_ERROR:
            raise exception.ThemisError(res, "Secure Session failed receiving")
        elif res < 0:
            return ""
        return ctypes.string_at(message, res)

    def is_established(self):
        return themis.secure_session_is_established(self.session_ctx) == 1

    def connect_request(self):
        req_size = ctypes.c_int(0)
        res = themis.secure_session_generate_connect_request(
            self.session_ctx, None, ctypes.byref(req_size))
        if res != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise exception.ThemisError(
                res, "Secure Session failed generating connect request")

        req_buffer = ctypes.create_string_buffer(req_size.value)
        res = themis.secure_session_generate_connect_request(
            self.session_ctx, ctypes.byref(req_buffer), ctypes.byref(req_size))
        if res != THEMIS_CODES.SUCCESS:
            raise exception.ThemisError(
                res, "Secure Session failed generating connect request")

        return ctypes.string_at(req_buffer, req_size)

    def wrap(self, message):
        send_message = ctypes.create_string_buffer(message)
        wrapped_message_length = ctypes.c_int(0)
        res = themis.secure_session_wrap(
            self.session_ctx, ctypes.byref(send_message), len(message), 0,
            ctypes.byref(wrapped_message_length))
        if res != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise exception.ThemisError(res, "Secure Session failed encrypting")

        wrapped_message = ctypes.create_string_buffer(
            wrapped_message_length.value)
        res = themis.secure_session_wrap(
            self.session_ctx, ctypes.byref(send_message), len(message),
            ctypes.byref(wrapped_message), ctypes.byref(wrapped_message_length))
        if res != THEMIS_CODES.SUCCESS:
            raise exception.ThemisError(res, "Secure Session failed encrypting")

        return ctypes.string_at(wrapped_message, wrapped_message_length)

    def unwrap(self, message):
        wrapped_message = ctypes.create_string_buffer(message)
        unwrapped_message_length = ctypes.c_int(0)
        res = themis.secure_session_unwrap(
            self.session_ctx, wrapped_message, len(message), 0,
            ctypes.byref(unwrapped_message_length))

        if res == THEMIS_CODES.SUCCESS:
            return SString(b"")

        if res != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise exception.ThemisError(res, "Secure Session failed decrypting")
        unwrapped_message = ctypes.create_string_buffer(
            unwrapped_message_length.value)
        res = themis.secure_session_unwrap(
            self.session_ctx, wrapped_message, len(message),
            ctypes.byref(unwrapped_message),
            ctypes.byref(unwrapped_message_length))
        rez = SString(ctypes.string_at(unwrapped_message,
                                       unwrapped_message_length))
        if res == THEMIS_CODES.SEND_AS_IS:
            rez.set_control()
        elif res != THEMIS_CODES.SUCCESS:
            raise exception.ThemisError(res, "Secure Session failed decrypting")

        return rez


class MemoryTransport(object):
    def __init__(self):
        self.message_list = deque()

    def send(self, message):
        self.message_list.append(message)

    def receive(self, buffer_length):
        return self.message_list.popleft()

    @abstractmethod
    def get_pub_key_by_id(self, user_id):
        raise NotImplementedError


class SimpleMemoryTransport(MemoryTransport):
    def __init__(self, expected_client_id, public_key):
        self._expected_client_id = expected_client_id
        self._public_key = public_key
        super(SimpleMemoryTransport, self).__init__()

    def get_pub_key_by_id(self, user_id):
        if user_id == self._expected_client_id:
            return self._public_key
        else:
            raise ThemisError(THEMIS_CODES.FAIL, "Incorrect user_id")


class sstring(SString):
    def __init__(self, *args, **kwargs):
        warnings.warn("sstring is deprecated in favor of SString.")
        super(sstring, self).__init__()


class ssession(SSession):
    def __init__(self, *args, **kwargs):
        warnings.warn("ssession is deprecated in favor of SSession.")
        super(ssession, self).__init__(*args, **kwargs)


class mem_transport(MemoryTransport):
    def __init__(self):
        warnings.warn(
            "mem_transport is deprecated in favor of MemoryTransport.")
        super(mem_transport, self).__init__()



#!/usr/bin/env python
# coding: utf-8
#
# Copyright (c) 2015 Cossack Labs Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import warnings
import ctypes
from ctypes.util import find_library
from enum import IntEnum

from . import exception as exception
from .exception import THEMIS_CODES

themis = ctypes.cdll.LoadLibrary(find_library("themis"))

themis.secure_comparator_get_result.restype = ctypes.c_int64

scomparator_create = themis.secure_comparator_create
scomparator_create.restype = ctypes.POINTER(ctypes.c_int)


class SCOMPARATOR_CODES(IntEnum):
    MATCH = 21
    NOT_MATCH = 22
    NOT_READY = 0


class SComparator(object):
    def __init__(self, shared_secret):
        self.session_ctx = ctypes.POINTER(ctypes.c_int)
        self.comparator_ctx = scomparator_create()
        if self.comparator_ctx is None:
            raise exception.ThemisError(THEMIS_CODES.FAIL,
                                        "Secure Comparator failed creating")
        res = themis.secure_comparator_append_secret(
            self.comparator_ctx,
            ctypes.byref(ctypes.create_string_buffer(shared_secret)),
            len(shared_secret))
        if res != THEMIS_CODES.SUCCESS:
            raise exception.ThemisError(
                THEMIS_CODES.FAIL, "Secure Comparator failed appending secret")

        self.comparation_complete = False

    def __del__(self):
        themis.secure_comparator_destroy(self.comparator_ctx)

    def begin_compare(self):
        req_size = ctypes.c_int(0)
        res = themis.secure_comparator_begin_compare(self.comparator_ctx, None,
                                                     ctypes.byref(req_size))
        if res != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise exception.ThemisError(
                res, "Secure Comparator failed making initialization message")
        req_buffer = ctypes.create_string_buffer(req_size.value)
        res = themis.secure_comparator_begin_compare(
            self.comparator_ctx, ctypes.byref(req_buffer),
            ctypes.byref(req_size))
        if res != THEMIS_CODES.SUCCESS and res != THEMIS_CODES.SEND_AS_IS:
            raise exception.ThemisError(
                res, "Secure Comparator failed making initialization message")
        return ctypes.string_at(req_buffer, req_size)

    def proceed_compare(self, control_message):
        c_message = ctypes.create_string_buffer(control_message)
        req_size = ctypes.c_int(0)
        res = themis.secure_comparator_proceed_compare(
            self.comparator_ctx, ctypes.byref(c_message), len(control_message),
            None, ctypes.byref(req_size))

        if res == THEMIS_CODES.SUCCESS:
            return ""

        if res != THEMIS_CODES.BUFFER_TOO_SMALL:
            raise exception.ThemisError(
                res, "Secure Comparator failed proceeding message")
        req_buffer = ctypes.create_string_buffer(req_size.value)
        res = themis.secure_comparator_proceed_compare(
            self.comparator_ctx, ctypes.byref(c_message), len(control_message),
            req_buffer, ctypes.byref(req_size))
        if res == THEMIS_CODES.SEND_AS_IS or res == THEMIS_CODES.SUCCESS:
            return ctypes.string_at(req_buffer, req_size)

        raise exception.ThemisError(
            res, "Secure Comparator failed proceeding message")

    def is_compared(self):
        return not (themis.secure_comparator_get_result(self.comparator_ctx) ==
                    SCOMPARATOR_CODES.NOT_READY)

    def is_equal(self):
        return (themis.secure_comparator_get_result(self.comparator_ctx) ==
                SCOMPARATOR_CODES.MATCH)

    def result(self):
        return themis.secure_comparator_get_result(self.comparator_ctx)


class scomparator(SComparator):
    def __init__(self, *args, **kwargs):
        warnings.warn("scomparator is deprecated in favor of SComparator.")
        super(scomparator, self).__init__(*args, **kwargs)



#
# Copyright (c) 2015 Cossack Labs Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import warnings
from enum import IntEnum


class THEMIS_CODES(IntEnum):
    NETWORK_ERROR = 2222
    BUFFER_TOO_SMALL = 14
    FAIL = 11
    SUCCESS = 0
    SEND_AS_IS = 1


class ThemisError(Exception):
    def __init__(self, error_code, message):
        self.error_code = error_code
        self.message = message

    def __str__(self):
        return repr("Themis error: {} -- {}".format(str(self.error_code),
                                                    self.message))


class themis_exception(ThemisError):
    def __init__(self, *args, **kwargs):
        warnings.warn("themis_exception is deprecated in favor of ThemisError.")
        super(themis_exception, self).__init__(*args, **kwargs)



#
# Copyright (c) 2015 Cossack Labs Limited
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import sys
from setuptools import setup


install_requires = []
if sys.version_info < (3, 4):
    install_requires.append('enum34')

setup(
    name='pythemis',
    version='0.11.0',

    description='',
    long_description=open("README.rst").read(),
    license='Apache 2.0',
    url='https://cossacklabs.com',

    author='CossackLabs',
    author_email='dev@cossacklabs.com',

    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: BSD",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],

    packages=["pythemis"],

    entry_points={},

    install_requires=install_requires,
)
